<?php
/**
 * Файл для класса описания поля грида, {@link mxl_dict_dbgridfield}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания поля грида.
 * 
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_dbgridfield{
	/**
	 * Список доступных типов данных для полей грида
	 * @var array of string
	 */
	public static $possibleDataTypes = array('int', 'string', 'float', 'date', 
			'datetime', 'edatetime');
	
	/**
	 * Функция проверки типа данных на его допустимость. В случае если тип является
	 * недопестимым выбрасывается исклчючение.
	 * @param string $dataType - проверяемый тип
	 * @throws {@link mxl_cfg_exceptions::$dict_dbgridfield_checkType}
	 * @return boolean
	 */
	public static function checkDataType($dataType){
		foreach (mxl_dict_dbgridfield::$possibleDataTypes as $dType)
			if ($dataType == $dType)
				return true;

		$errMess = mxl_cfg_exceptions::$dict_dbgridfield_checkType;
		if (strpos($errMess, '#VTINS') !== false)
			$errMess = str_replace('#VTINS', $dataType, $errMess);
					
		throw new Exception($errMess);
		return false;
	}
	
	/**
	 * Имя поля.
	 * @var string
	 */
	private $name;
	
	/**
	 * Заголовок поля
	 * @var string
	 */
	private $caption;
	
	/**
	 * Ширина поля при выводе в визуальную часть
	 * @var int
	 */
	private $width;
	
	/**
	 * Атрибуты поля 
	 * @var mxl_dict_fieldattribs
	 */
	private $attribs;
	
	/**
	 * Масска ввода
	 * @var string
	 */
	private $maskRe;
	
	/**
	 * Тип данных. Поддерживаются: string, int, date, time, edatetime, .
	 * @var string
	 */
	private $dataType;
	
	/**
	 * Размер (длинна) поля, актуальна для строковых типов.
	 * @var int
	 */
	private $dataLength;
	
	/**
	 * Полный текст связи (отношения)
	 * @var string
	 */
	private $relFull;
	
	/**
	 * Имя таблицы к которой строится связь. 
	 * @var string
	 */
	private $relTable;
	
	/**
	 * Синоним таблицы к которой строится связь
	 * @var string
	 */
	private $relSyn;
	
	/**
	 * Поле в таблице к которому строится связь.
	 * @var string
	 */
	private $relField;
	
	/**
	 * Условие связи для ключевых полей.
	 * Например:
	 * a.fk_id=b.link_table_primary_key_id
	 * @var string
	 */
	private $rel;
	
	/**
	 * Имя комбобокса, отвечающего за выбор значений.
	 * @var unknown_type
	 */
	private $cfgComboName;
	/**
	 * @var integer
	 * @values:
	 *   0 - Undefined update Link
	 *   1 - Local     update Link (on Client) - возможно только если для данной 
	 *                 link таблицы есть всего лишь одно |NU| поле
	 *   2 - Remote    update Link (on Server)
	 */
	private $cfgComboType;
	
	/**
	 * Ширина списка для комбобокса. Если не установлена то ширина равна ширине
	 * поля
	 * @var integer
	 */
	private $listWidth;
	
	/**
	 * Формат отображения полей даты
	 * @var string
	 */
	private $formatDate;
	
	/**
	 * Формат отображения полей времени
	 * @var string
	 */
	private $formatTime;

	/**
	 * Формат отображения числовых полей
	 * @var string
	 */
	private $formatNumber;
	
	/**
	 * Конструктор класса.
	 * @param $name - имя поля
	 * @param $caption - заголовок поля
	 * @param $width - ширина отображения в визуальной части
	 * @param $dataType - тип данных
	 * @param $dataLength - длинна (ширины) данных, актуально для строковых типов
	 * @param $attribs - макрос атрибута
	 * @param $maskRe - маска ввода
	 * @param $relation - связь
	 * @param $cfgComboName - макрос комбо для выбора значений
	 * @return mxl_dict_dbgridfield
	 */
	function __construct($name, $caption, $width, $dataType, $dataLength = 0, 
			$attribs = '', $maskRe = '', $relation = '', $cfgComboName = ''){
		
		$this->name         = strtolower($name);
		$this->caption      = $caption;
		$this->width        = $width;
		$this->attribs      = new mxl_dict_fieldattribs($attribs);
		
		$this->maskRe       = $maskRe;
		try{
			if (mxl_dict_dbgridfield::checkDataType($dataType))
				$this->dataType = $dataType;
		} catch (Exception $e){
			$errMess = mxl_cfg_exceptions::$dict_dbgridfield_checkType_fieldName;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $name, $errMess);
			
			throw new Exception($errMess .' '. $e->getMessage());
		}
				
		$this->dataLength   = $dataLength;
		
		$this->relFull      = strtolower($relation);
		$this->relTable     = '';
		$this->relSyn       = '';
		$this->relField     = '';
		$this->rel          = '';
		
		$this->setRelation($relation, $cfgComboName);
		
		$this->formatDate  = mxl_cfg_base::$format__date;
		$this->formatTime  = mxl_cfg_base::$format__time;
		$this->formatNumber = '';
	}
	
	/**
	 * Функция парсит макрос отношения (связи) и заполняет свойства связей.
	 * @param $relation - макрос связи
	 * @param $cfgComboName - комбо для выбора значений
	 */
	public function setRelation($relation = '', $cfgComboName = ''){
		if ($relation != ''){
			$relArray = mxl_utils_str::exclude_repetitions($relation, ';', false);
			
			$size = sizeof($relArray);
			
			$i = 0;
			foreach ($relArray as $r){
				switch ($i){
					case 0:
						if     ($size == 4)	$this->relTable = strtolower($r);
						elseif ($size == 2)	$this->relSyn   = strtolower($r);
						elseif ($size == 1)	$this->rel      = strtolower($r);
						break;
					case 1:
						if     ($size == 4)	$this->relSyn   = strtolower($r);
						elseif ($size == 2)	$this->relField = strtolower($r);
						break;
					case 2:
						if     ($size == 4)	$this->relField = strtolower($r);
					case 3:
						if     ($size == 4)	$this->rel      = strtolower($r);
						break;
				}
				$i++;
			}
		}
		
		$this->cfgComboName = $cfgComboName;
		
		$this->cfgComboType = 0;
		if ($cfgComboName)
			if ($pos = strpos($cfgComboName, '@') !== false) {
				$this->cfgComboName = substr($cfgComboName, $pos, strlen($cfgComboName));
				$this->cfgComboType = 2;
			} else
				$this->cfgComboType = 1;
	}
	
	/**
	 * Возвращает имя поля
	 * @return string
	 */
	public function getName(){
		return $this->name;
	}

	/**
	 * Возвращает заголовок поля
	 * @return string
	 */
	public function getCaption(){
		return $this->caption;
	}
    
    /**
	 * Устанавливает заголовок поля
	 * @param string $caption - заголовок поля
	 */
	public function setCaption($caption){
		$this->caption = $caption;
	}

	/**
	 * Возвращает тип данных. Может быть: string, int, date, time, edatetime, .
	 * @return string
	 */
	public function getDataType(){
		return $this->dataType;
	}
	
	/**
	 * Возвращает размер (длинну) данных. Актуально для строковых полей.
	 * @return int
	 */
	public function getDataLength(){
		return $this->dataLength;
	}
	
	/**
	 * Возвращает ширину поля, для отобрадения в визуальной части
	 * @return int
	 */
	public function getWidth(){
		return $this->width;
	}
	
	/**
	 * Возвращает маску ввода поля
	 * @return string
	 */
	public function getMaskRe(){
		return $this->maskRe;
	}
	
	/**
	 * Возвращает синоним таблицы к которой строится связь.
	 * @return string
	 */
	public function getRelSyn(){
		return $this->relSyn;
	}
	
	/**
	 * Возращает имя поля к которому строится связь
	 * @return string
	 */
	public function getRelField(){
		return $this->relField;
	}
	
	/**
	 * Возвращает имя таблицы к кторой строится связь
	 * @return string
	 */
	public function getRelTable(){
		return $this->relTable;
	}
	
	/**
	 * Возвращает условие по которому строится связь
	 * @return string
	 */
	public function getRel(){
		return $this->rel;
	}
	
	/**
	 * Функция для извлечения ид поля из связи, извлекаем поле до '='
	 * так как нужно поле из связи, которое в таблице.
	 * @param $rel
	 * @return string
	 */
	public function getRelIdFieldFromRel(){
		$rel1 = $this->rel;
		
		if ($rel1 != ''){
			if (strpos($rel1, '=') > 0){
				$exclude = mxl_utils_str::exclude_repetitions($rel1, '=', false);
				$rel1 = $exclude[0];//0 - до 1 - после
			}
			$exclude = mxl_utils_str::exclude_repetitions($rel1, '.', false);
			
			return $exclude[1];
		}
		return '';
	}

	/**
	 * Возвращает класс атрибутов поля
	 * @return mxl_dict_fieldattribs
	 */
	public function getAttribs() {
		return $this->attribs;
	}
	
	/**
	 * Возвращает имя комбо для выбора значений
	 * @return string
	 */
	public function getCfgComboName(){
		return $this->cfgComboName;
	}
	
	/**
	 * Возвращает тип комбо для выбора значений.
	 * Тип может быть:
	 *   0 - не определенный тип обновления полей
	 *   1 - Local     update Link (on Client) - возможно только если для данной 
	 *			link таблицы есть всего лишь одно |NU| поле
	 *   2 - Remote    update Link (on Server) - запрос отправляется на сервер 
	 *   		для получения значений связанных полей
	 * @return int
	 */
	public function getCfgComboType(){
		return $this->cfgComboType;
	}
	
	/**
	 * Метод установки формата отображения даты.
	 * @param string $formatDate
	 */
	public function setFormatDate($formatDate){
		$this->formatDate = $formatDate;
	}
	
	/**
	 * Метод получения формата отображения даты.
	 * @return string
	 */
	public function getFormatDate(){
		return $this->formatDate;
	}

	/**
	 * Метод установки формата отображения времени.
	 * @param string $formatTime
	 */
	public function setFormatTime($formatTime){
		$this->formatTime = $formatTime;
	}
	
	/**
	 * Метод получения формата отображения времени.
	 * @return string
	 */
	public function getFormatTime(){
		return $this->formatTime;
	}
	
	/**
	 * Метод установки формата отображения числовых полей.
	 * @param string $formatNumber
	 */
	public function setFormatNumber($formatNumber){
		$this->formatNumber = $formatNumber;
	}
	
	/**
	 * Метод получения формата отображения числовых полей.
	 * @return string
	 */
	public function getFormatNumber(){
		return $this->formatNumber;
	}
	
	/**
	 * Ширина списка для комбобокса. Если не установлена то ширина равна ширине
	 * поля
	 * @return int
	 */
	public function getListWidth(){
		return $this->listWidth;
	}
	
	/**
	 * Ширина списка для комбобокса. Если не установлена то ширина равна ширине
	 * поля
	 * @param int $listWidth - значение ширины списка
	 */
	public function setListWidth($listWidth){
		$this->listWidth = $listWidth;
	}
}
?>