<?php
/**
 * Файл для класса описания "итога по гриду", {@link mxl_dict_dbgridtotalgrid}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания "итога по гриду".
 * 
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_dbgridtotalgrid{
	/**
	 * Имя итога, должно совпадать с именем поля в гриде.
	 * @var string
	 */
	private $name;
	/**
	 * Заголовок итога. Отображается перед итогом в шапке грида.
	 * @var string
	 */
	private $caption;
	/**
	 * Тип итога, может принимать значения: 'sum', 'count', 'count*', 'min', 
	 * 'max'.
	 * @var string
	 */
	private $type;
	
	/**
	 * Проверка правильности типа итога.
	 * Если тип проходит проверку то возвращается значение true, если не 
	 * проходит то выбрасывается исключение и возвращается false. 
	 * 
	 * @param $type
	 * @throws {@link mxl_cfg_exceptions::$dict_dbgridtotalgrid_checkType}
	 * @return boolean
	 */
	private function checkType($type){
		$type = strtolower($type);
		
  		if (($type = 'sum') || ($type = 'count') || ($type = 'count*') 
  				|| ($type = 'min') || ($type = 'max'))
  			return true;
  		else {
  			$errMess = mxl_cfg_exceptions::$dict_dbgridtotalgrid_checkType;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $type,	$errMess);
			throw new Exception($errMess);
  			return false;
  		} 
	}
	
	/**
	 * Конструктор класса "Итоги по гриду". Создает и инициализирует объект.
	 * Перед созданием объекта происходит проверка правильности задания типа
	 * итога, если она не проходит то выбрасывается исключение.
	  
	 * @param $name - Имя итога, должно совпадать с именем поля в гриед
	 * @param $type - тип итога, может принимать значения: 'sum', 'count', 
	 * 		'count*', 'min', 'max'
	 * @param $caption - Заголовок итога, отображается в заголовке грида, перед
	 * 		итогом.
	 * @throws {@link mxl_cfg_exceptions::$dict_dbgridtotalgrid_checkType}
	 */
	function __construct($name, $type, $caption){
		if (!$this->checkType($type))
			exit;
			
		$this->name     = strtolower($name);
		$this->caption  = $caption;
		$this->type     = $type;
	}
	
	/**
	 * Возвращает имя итога (совпадает с именем поля, по которому строится итог)
	 * @return string
	 */
	public function getName(){
		return $this->name;
	}

	/**
	 * Возвращает заголовок итога
	 * @return string
	 */
	public function getCaption(){
		return $this->caption;
	}
	
	/**
	 * Возвращает тип итога
	 * @return string
	 */
	public function getType(){
		return $this->type;
	}
}
?>