<?php
/**
 * Файл для класса правил валидации секции dbgrid, {@link mxl_dict_validator_rulesdbgrid}.
 * 
 * @package mxl_dict_validator
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс правил валидации секции dbgrid.
 * Правила секции dbgrid предназначены для проверки значений гридов.
 * Запустить проверку правильности данных можно с помощью статической функции 
 * {@link mxl_dict_validator_rulesdbgrid::execute()}.
 * 
 * @package mxl_dict_validator
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_validator_rulesdbgrid {
	
	/**
	 * Статическая функция вызывает методы для проверки массива данных $row в соответствии 
	 * с правилами $rules и метаданными $dbgrid. Возвращает true в случае успешной 
	 * проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param array $rules - массив правил: ключевое значение определяет метод обработки,
	 * 		значение по ключу содержит правило проверки 
	 * @param array $row - массив данных для проверки
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	public static function execute ($rules, $row, mxl_dict_dbgrid $dbgrid){
		foreach ($rules as $ruleName => $rule) {
			if ($ruleName == 'required'){
				$result = mxl_dict_validator_rulesdbgrid::required($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			} if ($ruleName == 'periodMember'){
				$result = mxl_dict_validator_rulesdbgrid::periodMember($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'checkType'){
				$result = mxl_dict_validator_rulesdbgrid::checkType($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'unique'){
				$result = mxl_dict_validator_rulesdbgrid::unique($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'uniqueLR'){
				$result = mxl_dict_validator_rulesdbgrid::uniqueLR($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'checkEMail'){
				$result = mxl_dict_validator_rulesdbgrid::checkEMail($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'checkLogin'){
				$result = mxl_dict_validator_rulesdbgrid::checkLogin($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'checkPassword'){
				$result = mxl_dict_validator_rulesdbgrid::checkPassword($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			} else if ($ruleName == 'match'){
				$result = mxl_dict_validator_rulesdbgrid::match($rule, $row, $dbgrid);
				if ($result !== true)
					return $result;
			}
			
		} 
		return true;
	}
	
	/**
	 * Метод проверки полей, необходимых к заполнению. Возвращает true в случае 
	 * успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';'.
	 * 		пример: 'paramToCheck1;paramToCheck2;...' 
	 * @param array $row - массив для проверки на наличие значений
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	public static function required($rule, $row, mxl_dict_dbgrid $dbgrid){
		$result = '';
		$fieldsToCheck = explode(';', $rule);
		foreach ($fieldsToCheck as $field){
			$field = trim($field);
            if ($field != '') {
                if ($dbgrid->getFieldByName($field)->getDataType() == 'edatetime'){
                    $fieldD = $field.'_date';
                    $fieldT = $field.'_time';
                    if ( ((!isset($row[$fieldD])) || ($row[$fieldD] == '')) )
                    	$result .= ($result != ''?',':'').'"'.
                        		$dbgrid->getFieldByName($field)->getCaption().'(D)"';
                    if ( ((!isset($row[$fieldT])) || ($row[$fieldT] == '')) )
                    	$result .= ($result != ''?',':'').'"'.
                        		$dbgrid->getFieldByName($field)->getCaption().'(T)"';
                } else {
                    if ( ((!isset($row[$field])) || ($row[$field] == '')) )
                    	$result .= ($result != ''?',':'').'"'.
                        		$dbgrid->getFieldByName($field)->getCaption().'"';
                }
            }
        }
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_required
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}

	
	/**
	 * Вспомогательный метод для разбора строки правила вида:
	 * 'min1<field1<=max1', 'field2<max2', 'field3>=min3' или т.п. 
	 * Возвращает массив параметров правила: 
	 * 		max - максимальная граница интервала, 
	 * 		min - минимальная граница интервала, 
	 * 		maxstrict - true, если неравенство при максимальном значении строгое,
	 * 		minstrict - true, если неравенство при минимальном значении строгое,
	 * 		field - название поля.
	 * Если один из параметров (max, min) не найден, возвращается null.
	 * 
	 * @param string $rule - правило для одного поля
	 * @return array
	 */
	private static function parseIntervalRule($rule) {
		$max       = null;
		$min       = null;
		$maxstrict = true;
		$minstrict = true;
		$field     = $rule;
		if (strpos($field, '<') !== false){
			$x = explode('<', $field);
			if (count($x) == 2) {
				$field = $x[0];
				$pos = strpos($x[1], '='); 
				if ($pos === false) {
					$max = $x[1] == '' ? null : $x[1];
				} else if ($pos == 0) {
					$buf = substr($x[1], 1);
					$max = $buf == '' ? null : $buf;
						$maxstrict = false;
				}
					unset($x);
			} else if (count($x) == 3) {
				$field = $x[1];
				$pos = strpos($field, '=');
				if ($pos === false) {
					$min = $x[0] == '' ? null : $x[0];	
				} else if ($pos == 0) {
					$field = substr($field, 1);
					$min = $x[0] == '' ? null : $x[0];
					$minstrict = false;
				}
				$pos = strpos($x[2], '='); 
				if ($pos === false) {
					$max = $x[2] == '' ? null : $x[2];
				} else if ($pos == 0) {
					$buf = substr($x[2], 1);
					$max = $buf == '' ? null : $buf;
					$maxstrict = false;
				}
				unset($x);
			}
		}
		if (strpos($field, '>') !== false){
			$x = explode('>', $field);
			if (count($x) == 2) {
				$field = $x[0];
				$pos = strpos($x[1], '='); 
				if ($pos === false) {
					$min = $x[1] == '' ? null : $x[1];
				} else if ($pos == 0) {
					$buf = substr($x[1], 1);
					$min = $buf == '' ? null : $buf;
					$minstrict = false;
				}
				unset($x);
			}
		}
		$result = array('max' => trim($max), 
				'min'         => trim($min), 
				'maxstrict'   => trim($maxstrict), 
				'minstrict'   => trim($minstrict),
				'field'       => trim($field) );
		return $result;		
	}
	
	/**
	 * Метод проверки принадлежности значений полей промежутку допустимых значений. 
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля с границами интервала передаются через ';'.
	 * 		Если указывается одна из границ интервала, то сперва записывается имя поля,
	 * 		затем знак соответствующего неравенства, а затем значение границы.
	 * 		Если указываются обе границы интервала допустимых значений, то они записываются
	 * 		последовательно: минимальная граница, знак неравенства, имя поля, знак неравенства,
	 * 		максимальная граница.
	 * 		Проверяемые неравенства могут быть строгими (>, <) или не строгими (>=, <=).
	 * 		пример: 'minvalue1<field1<=maxvalue1;field2<maxvalue2;field3>=minvalue3;...'
	 * @param array $row - массив для проверки принадлежности значений интервалу 
	 * 		допустимых значений
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	public static function periodMember($rule, $row, mxl_dict_dbgrid $dbgrid){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field){
			$arr       = mxl_dict_validator_rulesdbgrid::parseIntervalRule($field);
			$max       = $arr['max'];
			$min       = $arr['min'];
			$maxstrict = $arr['maxstrict'];
			$minstrict = $arr['minstrict'];
			$field     = $arr['field'];
			if ($field != '' && isset($row[$field])) {
				$caption   = $dbgrid->getFieldByName($field)->getCaption();
				if (!is_null($min)) {
					if ($minstrict && $row[$field] <= $min) {
						$result .= ($result != ''?',':'').'"'.$caption.'" > '.$min;
					} else if (!$minstrict && $row[$field] < $min) {
						$result .= ($result != ''?',':'').'"'.$caption.'" >= '.$min;
					}
				}
				if (!is_null($max)) {
					if ($maxstrict && $row[$field] >= $max) {
						$result .= ($result != ''?',':'').'"'.$caption.'" < '.$max;
					} else if (!$maxstrict && $row[$field] > $max) {
						$result .= ($result != ''?',':'').'"'.$caption.'" <= '.$max;	
					}
				} 
			}
				
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_periodMember
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	/**
	 * Вспомогательный метод для проверки соответствия значения поля его типу и 
	 * парметрам (длина строки, точность вещественных типов). 
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $field - имя поля
	 * @param string $value - значение поля
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	private static function typeIsValid($field, $value, mxl_dict_dbgrid $dbgrid) {
		$fi = $dbgrid->getFieldByName($field);
		$result = true;
		switch ($fi->getDataType()) {
			case 'int':
				$result = is_int($value);
				break;
			case 'boolean':
				$result = is_int($value);
				break;
			case 'double' | 'float':
				$result = is_int($value) || is_float($value);
				if ($result === true) {
					$x = explode('.', $value);
					$buf = $x[1];
					if (strlen($buf) > $fi->getDataLength()) 
						$result = ' ('.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkType1
							.' {*}.{'.$fi->getDataLength().'})';
				} else {
					$result = '';
				}
				break;
			case 'date':
				$result = is_string($value);
				break;
			case 'edatetime':
				$result = is_string($value);
				break;
			case 'string':
				$result = is_string($value);
				if ($result === true) {
					if (strlen($value) > $fi->getDataLength()) 
						$result = ' ('.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkType2
							.$fi->getDataLength().')';
				} else {
					$result = '';
				} 
				break;
		}
		return $result;		
	}
	
	/**
	 * Метод проверки типа данных и параметров полей. Возвращает true в случае 
	 * успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';'.
	 * 		пример: 'paramToCheck1;paramToCheck2;...' 
	 * @param array $row - массив для проверки на наличие значений
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	public static function checkType($rule, $row, mxl_dict_dbgrid $dbgrid){
		$result = '';
		$fieldsToCheck = explode(';', $rule);
		foreach ($fieldsToCheck as $field){
			$field = trim($field);
			if ($field != '' && isset($row[$field])) {
				$err = mxl_dict_validator_rulesdbgrid::typeIsValid($field, $row[$field], $dbgrid);
				if ($err !== true) {
					$result .= ($result != ''?',':'').'"'
						.$dbgrid->getFieldByName($field)->getCaption().'" : "'
						.$dbgrid->getFieldByName($field)->getDataType()
						.'"'.$err;	
				}	
			}	
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_checkType
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	/**
	 * Метод проверки уникальности значений полей. Возвращает true в случае 
	 * успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';'.
	 * 		пример: 'paramToCheck1;paramToCheck2;...' 
	 * @param array $row - массив для проверки на наличие значений
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	public static function unique($rule, $row, mxl_dict_dbgrid $dbgrid){
		$result = '';
		$fieldsToCheck = explode(';', $rule);
		foreach ($fieldsToCheck as $field){
			if (($field != '') && isset($row[$field]) && $row[$field] != '') {
				$mxl_ldbs = new mxl_core_logicbases();
				$mxl_db = $mxl_ldbs->getLogicBaseInst($dbgrid->getLogicBaseName());
				$sql = 'SELECT '.$field.', ' . $dbgrid->getPrimKey() . ' FROM '.$dbgrid->getTableName(); //ошибка
				$result0 = $mxl_db->sql_query($sql);
				while ($row0 = $mxl_db->sql_fetchrow($result0)) {
					if ($row[$field] == $row0[$field] &&
						$row[$dbgrid->getPrimKey()] != $row0[$dbgrid->getPrimKey()]) {
						$result .= ($result != ''?',':'').'"'.
							$dbgrid->getFieldByName($field)->getCaption().'"';
						continue;
					}
				}
				$mxl_db->sql_freeresult($result0);
			}
				
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_unique
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}

	
	
	
	
	/**
	 * Метод проверки уникальности комбинаций значений полей . Возвращает true в случае 
	 * успешной проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';', комбинации полей разледяются ':'.
	 * 		пример: 'paramToCheck1:withCheck1;paramToCheck2:withCheck2;...' 
	 * @param array $row - массив для проверки на наличие значений
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	public static function uniqueLR($rule, $row, mxl_dict_dbgrid $dbgrid){
		$result = '';
		$fieldsToCheck = explode(';', $rule);
		foreach ($fieldsToCheck as $field){
			//неообходимо ввести поля, относительно друг друга, которые должны быть уникальны
			//примем, что при проверке на уникальность с первичным ключем, первичный ключ может не передаваться 
			//в массиве переданных данных, т.е. при проверке на первичный ключ, записи как таковой не должно быть
			//в базе. Также примем, что поле, которое должно быть уникально, идет первым, поле на которое опираемся - вторым
			if ($field != ''){
			$fieldsLR = explode(':', $field);
			$fieldL = $fieldsLR[0];
			$fieldR = $fieldsLR[1];
			}
			if (($field != '') && isset($row[$fieldL]) && $row[$fieldL] != '') {
				$mxl_ldbs = new mxl_core_logicbases();
				$mxl_db = $mxl_ldbs->getLogicBaseInst($dbgrid->getLogicBaseName());
				$sql =	'SELECT '.$fieldL.', ' . $fieldR 
						.' FROM '.$dbgrid->getTableName()
						.' WHERE ' . $fieldL . '=' . $row[$fieldL]
						.' AND ' . $fieldR . '=' . $row[$fieldR];
				$result0 = $mxl_db->sql_query($sql);
				while ($row0 = $mxl_db->sql_fetchrow($result0)) {
					if ($row[$fieldL] == $row0[$fieldL] &&
						$row[$fieldR] == $row0[$fieldR]) {
						$result .= ($result != ''?',':'').'"'.
							$dbgrid->getFieldByName($fieldL)->getCaption().'"';
						continue;
					}
				}
				$mxl_db->sql_freeresult($result0);
			}
				
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_unique
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	
	
	/**
	 * Метод валидации e-mail адреса. Возвращает true в случае успешной 
	 * проверки, иначе сообщение о нарушении правила.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';'.
	 * 		пример: 'paramToCheck1;paramToCheck2;...' 
	 * @param array $row - массив для проверки на наличие значений
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	public static function checkEMail($rule, $row, mxl_dict_dbgrid $dbgrid){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field){
			$field = trim($field);
			if ($field != '' && isset($row[$field]) &&
					!preg_match('/^[a-z][-\._a-z0-9]*@[a-z][-\._a-z0-9]*\.[a-z0-9]{2,5}$/i', $row[$field])) {
				$result .= ($result != ''?',':'').'"'.$dbgrid->getFieldByName($field)->getCaption().'"';
			}
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_checkEMail
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}
	
	/**
	 * Метод валидации логина.
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * Логин может содержать латинские символы, цифры, '_', '.', '-'. Первый символ 
	 * логина должен быть буквой или цифрой.
	 * По умолчанию проверяется, чтобы в логине было не менее 1 символа, но не более 20,
	 * но границы допустимых размеров строки логина могут быть заданы правилом.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля с границами интервала допустимой длины строки передаются через ';'.
	 * 		Если указывается одна из границ интервала, то сперва записывается имя поля,
	 * 		затем знак соответствующего неравенства, а затем значение границы.
	 * 		Если указываются обе границы интервала допустимых значений, то они записываются
	 * 		последовательно: минимальная граница, знак неравенства, имя поля, знак неравенства,
	 * 		максимальная граница.
	 * 		Проверяемые неравенства могут быть строгими (>, <) или не строгими (>=, <=).
	 * 		пример: 'field1;field2>=minlength2;minlength3<field3<=maxlength3;...'
	 * @param array $row - массив для валидации логина
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
		public static function checkLogin($rule, $row, mxl_dict_dbgrid $dbgrid){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field) {
			$arr = mxl_dict_validator_rulesdbgrid::parseIntervalRule($field);
			$field = $arr['field'];
			if (is_null($arr['max'])) {
				$maxlength = $dbgrid->getFieldByName($field)->getDataLength();
				$maxstrict = false;
			} else {
				$maxlength = $arr['max'];
				$maxstrict = $arr['maxstrict'];
			}
			if (is_null($arr['min'])) {
				$minlength = 1;
				$minstrict = false;	
			} else {
				$minlength = $arr['min'];
				$minstrict = $arr['minstrict'];
			}
			if ($field != '' && isset($row[$field])) {
				$caption = $dbgrid->getFieldByName($field)->getCaption();
			if ($minstrict && strlen($row[$field]) <= $minlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkLogin1.'> '
						.$minlength;
				} else if (!$minstrict && strlen($row[$field]) < $minlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkLogin1.'>= '
						.$minlength;
				} 
				if ($maxstrict && strlen($row[$field]) >= $maxlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkLogin1.'< '
						.$maxlength;
				} else if (!$maxstrict && strlen($row[$field]) > $maxlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkLogin1.'<= '
						.$maxlength;
				}
				if (!preg_match('/^[-\._a-z0-9]*$/i', $row[$field])) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkLogin2;
				} else if (!preg_match('/^[a-z0-9][-\._a-z0-9]*$/i', $row[$field])) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkLogin3;
				}
			}
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_checkLogin
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}

	/**
	 * Метод валидации пароля.
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * Пароль может содержать любые символы, кроме кириллицы и спецсимволов (пробел, табуляция)
	 * По умолчанию пароль может быть пустым, но не может содержать более 20 символов,
	 * но границы допустимых размеров строки пароля могут быть заданы правилом.
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля с границами интервала допустимой длины строки передаются через ';'.
	 * 		Если указывается одна из границ интервала, то сперва записывается имя поля,
	 * 		затем знак соответствующего неравенства, а затем значение границы.
	 * 		Если указываются обе границы интервала допустимых значений, то они записываются
	 * 		последовательно: минимальная граница, знак неравенства, имя поля, знак неравенства,
	 * 		максимальная граница.
	 * 		Проверяемые неравенства могут быть строгими (>, <) или не строгими (>=, <=).
	 * 		пример: 'field1;field2>=minlength2;minlength3<field3<=maxlength3;...'
	 * @param array $row - массив для валидации пароля
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида
	 * @return boolean | string
	 */
	public static function checkPassword($rule, $row, mxl_dict_dbgrid $dbgrid){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		foreach ($fieldsToCheck as $field) {
			$arr = mxl_dict_validator_rulesdbgrid::parseIntervalRule($field);
			$field = $arr['field'];
			if (is_null($arr['max'])) {
				$maxlength = $dbgrid->getFieldByName($field)->getDataLength();
				$maxstrict = false;
			} else {
				$maxlength = $arr['max'];
				$maxstrict = $arr['maxstrict'];
			}
			if (is_null($arr['min'])) {
				$minlength = 0;
				$minstrict = false;	
			} else {
				$minlength = $arr['min'];
				$minstrict = $arr['minstrict'];
			}
			if ($field != '' && isset($row[$field])) {
				$caption = $dbgrid->getFieldByName($field)->getCaption();
				if ($minstrict && strlen($row[$field]) <= $minlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkPassword1.'> '
						.$minlength;
				} else if (!$minstrict && strlen($row[$field]) < $minlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkPassword1.'>= '
						.$minlength;
				} 
				if ($maxstrict && strlen($row[$field]) >= $maxlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkPassword1.'< '
						.$maxlength;
				} else if (!$maxstrict && strlen($row[$field]) > $maxlength) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkPassword1.'<= '
						.$maxlength;
				}
				if (preg_match('/['.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkPassword2.']|.\s/i', 
						$row[$field])) {
					$result .= ($result != ''?',':'').'"'.$caption.'" '
						.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_checkPassword3;
				}
			}
		}
		if ($result != '')
			$result = '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_checkPassword
					.':<br>'.$result.'.';
		else
			$result = true;
		return $result;
	}

	/**
	 * Метод валидации по регулярному выражению.
	 * Возвращает true в случае успешной проверки, иначе сообщение о нарушении правила.
	 * Управляющие символы, используемые в регулярном выражении, должны быть 
	 * экранированы. Экранируются: одиночная кавычка ('), двойная кавычка ("), 
	 * обратный слэш (\) и NULL(байт NULL).
	 * 
	 * @param string $rule - параметр для передачи правила проверки, проверяемые
	 * 		поля передаются через ';'. В правиле через ':' указывается имя поля и 
	 * 		регулярное выражение. Если необходимо указать сообщение, которое будет 
	 * 		возвращаться в случае неудачной проверки, то параметр задается через ':!'
	 * 		после регулярного выражения. Чтобы вставить в сообщение ссылку на поле, 
	 * 		в нужном месте сообщения включите %f% (см. пример)
	 * 		пример: 'field1:match1;field2:match2:!Сообщение для поля %f%.;...' 
	 * @param array $row - массив данных для валидации по регулярному выражению
	 * @param mxl_dict_dbgrid $dbgrid - метаданные грида 
	 * @return boolean | string
	 */
	public static function match($rule, $row, mxl_dict_dbgrid $dbgrid){
		$fieldsToCheck = explode(';', $rule);
		$result = '';
		$result1 = '';
		$result2 = '';
		foreach ($fieldsToCheck as $field) {
			$caption = '';
			$match   = '';
			$message = '';
			if (strpos($field, ':') !== false){
				$x = explode(':', $field);
				if (count($x) == 3 && strpos(trim($x[2]), '!') == 0) {
					$field = trim($x[0]);
					$match = trim($x[1]);
					$message = substr(trim($x[2]), 1, strlen($x[2]));
					unset($x);	
				} else if (count($x) == 2) {
					$field = trim($x[0]);
					$match = trim($x[1]);
					unset($x);	
				} 
			}
			if ($field != '' && isset($row[$field])) {
				$caption = $dbgrid->getFieldByName($field)->getCaption();
				if (!preg_match($match, $row[$field])) {
					if ($message != '') {
						$result1 .= ($result1 != ''?'<br>':'')
							.str_replace("%f%", '"'.$caption.'" ', $message);
						$message = '';
					} else {
						$result2 .= ($result2 != ''?',':'').'"'.$caption.'" '
							.mxl_cfg_exceptions::$dict_validator_rulesdbgrid_match
							.$match;	
					}
				}
			}
		}
		if ($result1 != '')
			$result .= '<br>'.$result1;
		if ($result2 != '')
			$result .= '<br>'.mxl_cfg_messages::$dict_validator_rulesdbgrid_match
					.':<br>'.$result2.'.';
		if ($result == '')
			$result = true;
		return $result;
	}
}
?>
