<?php
/**
 * Файл описывающий класс работы с расширенным типом даты времени, {@link mxl_utils_edatetime}.
 * @package mxl_utils
 * @author merlin <merlin@ersys.ru>, Alexander Garashko <i.love.alex@mail.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс для работы с расширенным типом даты времени.
 * 
 * @package mxl_utils
 * @author merlin <merlin@ersys.ru>, Alexander Garashko <i.love.alex@mail.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_utils_edatetime{
	/**
	 * Функция заменяет одно поле с типом edatetime и именем $name на два поля
	 * даты и времени с именами $name.'_date' и $name.'_time' соответственно.  
	 * @param $r - ассоциированный массив, в котором требуется произвести замену.
	 * @param $name - поле с типом edatetime которое необходимо разложить на два поля.
	 */
	public static function excludeFromRow(&$r, $name) {
		if ($r)	{
			$r1 = array();
			foreach ($r as $n => $v) {
				if ($n != $name)
					$r1[$n] = $v;
				else {
					$date_val = '';
					$time_val = '';
					if ($v)	{
						$date_val = strftime('%m/%d/%Y',	$v);
						$time_val = strftime('%H:%M',		$v);
					}
					$r1[$n.'_date'] = $date_val;
					$r1[$n.'_time'] = $time_val;
				}
			}
			$r = $r1;
		}
	}
	
	/**
	 * Функция осуществляет обратное преобразование двух полей с именами 
	 * $name.'_date' и $name.'_time' в одно поле содержащие данные даты и времени
	 * и трактующиеся как edatetime 
	 * @param $r - ассоциированных массив, в котром необходимо осуществить 
	 * 		преобразование
	 * @param $name - имя поля для которого необходимо осуществить преобразование
	 */
	public static function includeToRow(&$r, $name) {
		$date_name = $name.'_date';
		$time_name = $name.'_time';
	
		if (!$r)
            return

		$r1 = array();
		foreach ($r as $n => $v) {
			if ($n != $date_name && $n != $time_name)
				$r1[$n] = $v;
			else if ($n == $date_name) {
				$d_intval = 0;
				$ds = $v;
				if ($ds) {
    				$ts = '';
        			if (isset($r[$time_name]))
            			$ts = $r[$time_name];
					$d_intval = mxl_utils_edatetime::datetime_to_int($ds, $ts);
                }
				$r1[$name] = (int) $d_intval;
			}
		}
		$r = $r1;
	}

	/**
	 * Смещение времени на $timeShift
	 * @param int $value
	 * @param float $timeShift
	 * @return int
	 */
	public static function applyTimeShift($value, $timeShift = 0) {
		if ($value)
			return (int) ($value + $timeShift*3600);
		else
			return $value;
	}
	
	/**
	 * Отмена смещения времени на $timeShift
	 * @param int $value
	 * @param float $timeShift
	 * @return int
	 */
	public static function cancelTimeShift($value, $timeShift = 0) {
		if ($value)
			return (int) ($value - $timeShift*3600);
		else
			return $value;
	}

    /**
     * Метод извлекает дату и время из строки и упаковывает в int
     * @param string $date_str - дата в виде строки
     * @param string $time_str - время в виде строки
     * @return int
     */
    public static function datetime_to_int($date_str, $time_str = ''){
		$date_int = 0;
		if ($date_str) {
			$d = date_parse($date_str);

    		$h = $m = $s = 0;
			if ($time_str) {
				$tt = strtotime($time_str . ':0');
        		$h  = strftime('%H', $tt);//date('H', $tt)
            	$m  = strftime('%M', $tt);//date('%M', $tt)
                $s  = strftime('%S', $tt);//date('%S', $tt)
			}
			$date_int = mktime($h, $m, $s, $d['month'], $d['day'], $d['year']);
		}
		return (int) $date_int;
    }
}
?>