<?php
/**
 * Файл для описания класса работы json, {@link mxl_utils_json}.
 * @package mxl_utils
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс для работы с json.
 * @package mxl_utils
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_utils_json{
	/**
	 * Начало динамического параметра
	 * @var string
	 */
	private static $begParam = '[:';
	/**
	 * Конец динамического параметра
	 * @var string
	 */
	private static $endParam = ':]';
	
	/**
	 * Начало сниппета
	 * @var string
	 */
	private static $begSnippet = '[[';
	/**
	 * Конец сниппета
	 * @var string
	 */
	private static $endSnippet = ']]';
	
	/**
	 * Функция анализирует последнюю ошибку при кодировании/декодировании json
	 * и выбрасывает исключение если таковая есть.
	 * 
	 * ВНИМАНИЕ!!!
	 * Поддержка фукнции определения ошибки начинается только с php v5.3.0
	 * 
	 * @throws Exception {@link mxl_cfg_exceptions::$json_depth},
	 * 		{@link mxl_cfg_exceptions::$json_ctrl_char}, 
	 * 		{@link mxl_cfg_exceptions::$json_syntax}  
	 */
	public static function testByErrors(){
		switch(json_last_error()) {
        	case JSON_ERROR_DEPTH:
            	throw new Exception(mxl_cfg_exceptions::$json_depth);
        	break;
        	case JSON_ERROR_CTRL_CHAR:
            	throw new Exception(mxl_cfg_exceptions::$json_ctrl_char);
        	break;
        	case JSON_ERROR_SYNTAX:
            	throw new Exception(mxl_cfg_exceptions::$json_syntax);
        	break;
        	case JSON_ERROR_NONE:
    	    break;
    	}
	} 
	
	/**
	 * Функция находит динамические переменные в json определении
	 * метаданных словаря, и заменяет их на значения в $params.
	 * @param string $json - json определение метаданных
	 * @return string
	 */
	public static function replaceParams($json, mxl_core_params $params){
		$jsonParams = mxl_utils_str::excludeSubStrings($json,
				mxl_utils_json::$begParam, mxl_utils_json::$endParam);
		
		if (count($jsonParams) < 1 )
			return $json;
		
		$params_res = array();

		foreach ($jsonParams as $key => $paramName){
			$paramName = mxl_utils_str::getSubString($paramName,
					mxl_utils_json::$begParam, mxl_utils_json::$endParam);
			try{
				$params_res[$key] = $params->getParamValue($paramName);
			} catch (Exception $e){
				throw new Exception($e->getMessage());
			} 
		} 
		
		$json = str_replace($jsonParams, $params_res, $json);
		
		return $json;
	}
	
	/**
	 * Функция находит динамические блоки (snippets) в json определении
	 * метаданных словаря, выполняет их и возвращает реультат в место вызова
	 * сниппета.
	 * @param string $json - json определение метаданных
	 * @return string
	 */
	public static function excecuteSnippet($json, $jsonFileDir, 
			mxl_core_params $params){
		$snippets = mxl_utils_str::excludeSubStrings($json,
				mxl_utils_json::$begSnippet, 
				mxl_utils_json::$endSnippet); 

		if (count($snippets) < 1 ) {
			return $json;
		}
		
		$snippets_res = array();

		foreach ($snippets as $key => $value){
			$snippet = mxl_utils_str::getSubString($value,
				mxl_utils_json::$begSnippet, 
				mxl_utils_json::$endSnippet);
			
			$snippet_res = mxl_utils_snippet::execute($snippet, 
					$jsonFileDir, $params);
			$snippets_res[$key] = $snippet_res; 
		}
		
		$json = str_replace($snippets, $snippets_res, $json);
		
		return $json;
	}
	
	/**
	 * Функция читает файлы в формате mxl и возвращает строку json описанную
	 * в файле, понимает комментарии в файле. Должен начинаться /* и заканчиваться * /
	 * (без разделяющего пробела).Нельзя использовать символы начала и окончания
	 * коментария в JSON строке.
	 * 
	 * Внимание! Строки надо кодировать строго, с учетом управляющих символов!
	 * Например: что бы строка приняла значение \n ее надо кодировать как: "\\n"
	 * 
	 * Имя параметра должно быть заключено в "", кроме целочисленных вещественных 
	 * типов и массивов.
	 * 
	 * @param $jsonFileName - имя файла, расширение должно быть mxl
	 * @param mxl_core_params $params - параметры метаданного
	 * @throws Exception {@link mxl_cfg_exceptions::$utils_json_readJSONFromFile_havent_param}
	 * @return json string
	 */
	public static function readJSONFromFile($jsonFileName, 
			mxl_core_params $params) {
		$fileExt = mxl_utils_files::getExtension($jsonFileName);
		if (mxl_cfg_base::$json_file_ext)
			if ( strtolower(trim($fileExt)) != mxl_cfg_base::$json_file_ext){
				throw new Exception('mxl_utils_json::readJSONFromFile '
						.mxl_cfg_exceptions::$utils_json_read_from_file_not_lib_ext);
			}
		$json = file_get_contents($jsonFileName);
		
		try{
			$json = mxl_utils_json::replaceParams($json, $params);
		
			$json = mxl_utils_json::excecuteSnippet($json, 
					dirname($jsonFileName), $params);
		
		} catch (Exception $e){
			$errMess = mxl_cfg_exceptions::$utils_json_readJSONFromFile_havent;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $jsonFileName, $errMess);
			
			throw new Exception($errMess .' '. $e->getMessage());
		} 
		
		/** TODO
		 * необходимо добавить обработчики которые будут:
		 * 1. Более интелектуально исключать коментарии из файла
		 */
		$commSt = strpos($json, '/*');
		while ($commSt !== false){
			$commEn = strpos($json, '*/');
			$json = substr_replace($json, '', $commSt, $commEn - $commSt + 2);
			$commSt = strpos($json, '/*');
		}
		$json = str_replace(array("\n", "\r", "\t"), array(' ', ' ', ' '), $json);
		
		return $json;
	}
	
	/**
	 * Функция читает json из файла и преобразует в асоциированный массив, если
	 * были ошибки при конвертации то выбрасывается исключение соответствующее
	 * ошибке.
	 * 
	 * ВНИМАНИЕ!!!
	 * Поддержка фукнции определения ошибки начинается только с php v5.3.0
	 * 
	 * @param $jsonFileName - имя файла, расширение должно быть mxl
	 * @param mxl_core_params $params - параметры метаданного
	 * @throws Exception {@link mxl_cfg_exceptions::$json_depth},
	 * 		{@link mxl_cfg_exceptions::$json_ctrl_char}, 
	 * 		{@link mxl_cfg_exceptions::$json_syntax}   
	 * @return array
	 */
	public static function readArrayFromFile($jsonFileName, 
			mxl_core_params $params){
		$json = mxl_utils_json::readJSONFromFile($jsonFileName, $params);
		$json_arr = json_decode($json, true);

		/**
		 * для php v5.3.0
		 * если версия ниже то закомментировать обращение к функции
		 */
		mxl_utils_json::testByErrors();
		
		return $json_arr;
	}
}
?>