<?php
/**
 * Файл для описания класса работы со строками и массивами строк, {@link mxl_utils_str}.
 * @package mxl_utils
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс для работы со строками и массивами строк.
 * @package mxl_utils
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_utils_str{
	/**
	 * Функция убирает повторяющиеся части из строки. Строка делется на части
	 * резделителем передающимся в параметрах функции.
	 * 
	 * Пример:
	 * <code>
	 *   mxl_utils_str::exclude_repetitions('mail_1@mailserver.com,'
	 *   		.'mail_2@mailserver.com,mail_3@mailserver.com,'
	 *   		.'mail_1@mailserver.com', ',', true);
	 * </code>
	 * возвратит:
	 * <code>
	 * 'mail_1@mailserver.com,mail_2@mailserver.com,mail_3@mailserver.com'
	 * </code>
	 *     
	 * @param $str - строка в которой необходимо исключить повторение
	 * @param $sym - разделитель
	 * @param $as_str - возвращает значение в виде: true - строки | false - массива
	 * @return string|array
	 */
	public static function exclude_repetitions($str, $sym, $as_str = false){
		$res = array();
		$lenSym = strlen($sym);
		while(strlen($str)){
			$pos = strpos($str, $sym);
			if ($pos){
				$path1 = substr($str, 0, $pos);
				$str   = substr($str, $pos+$lenSym, strlen($str)-$lenSym+1);
			} else {
				$path1 = $str;
				$str = '';
			}
			$res[] = $path1;
		}
		$res = array_unique($res);
		if($as_str)	{
			$fin = '';
			foreach ($res as $tmp){
				if ($fin)
					$fin .= $sym;
				$fin .=	$tmp;
			}
			return $fin;
		} else {
			return $res;
		}
	}
	
	/**
	 * Фукнция для получения части строки с порядковым номером $wordN.
	 * Внимание! 
	 * Перед определением порядкового номера из строки исключаются повторяющиеся
	 * части.
	 * @param $str - строка
	 * @param $wordN - порядковый номер
	 * @param $sym - символ (строка) разделителя на части
	 * @return string
	 */
	public static function getNWord($str, $wordN, $sym){
		$list = mxl_utils_str::exclude_repetitions($str, $sym);
		return $list[$wordN];
	}
	
	/**
	 * Функция извлекает все подстроки из строки $string, которые начинаются с
	 * $subStrBeg и заканчиваются $subStrEnd. 
	 * @param strign $string - строка в которой происходит поиск
	 * @param string $subStrBeg - строка с которой начинается субстрока
	 * @param string $subStrEnd - строка с которой заканчивается субстрока
	 * @param boolean $asArray - вернуть результат в качестве массива,
	 * 		в противном случае в качестве строки
	 * @param string $delimeter - разделитель строк для случая возвращения 
	 * 		в качестве строки ($asArray == false)
	 * @return string || array
	 */
	public static function excludeSubStrings($string, $subStrBeg, $subStrEnd,
			$asArray = true, $delimeter = ''){
		$subStrings = array();
		$lenStrBeg = strlen($subStrBeg);
		
		$st = strpos($string, $subStrBeg);
		while ($st !== false){
			$en = strpos($string, $subStrEnd, $st);
			$subStrings[] = substr($string, $st, $en - $st + $lenStrBeg);
			$st = strpos($string, $subStrBeg, $st + $lenStrBeg);
		}
		
		if (!$asArray){
			$output = '';
			foreach($subStrings as $value)
				$output .= $value.$delimeter;
			return $output;
		} else
			return $subStrings;
	}
	
	/**
	 * Функция для получения текста между $subStrB и $subStrE
	 * @param strign $string - строка в которой происходит поиск
	 * @param string $subStrB - строка с которой начинается субстрока
	 * @param string $subStrE - строка с которой заканчивается субстрока
	 * @return string
	 */
	public static function getSubString($string, $subStrB, $subStrE){
		$lenStrB = strlen($subStrB);
		$lenStrE = strlen($subStrE);
		return  substr($string, strpos($string, $subStrB) + $lenStrB, 
				strpos($string, $subStrE) - strpos($string, $subStrB) - $lenStrE);
	}
}
?>