<?php
/**
 * Файл для описания класса визуального компонента комбобокс, 
 * {@link mxl_widget_combobox}.
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания виждета комбобокса.
 * 
 * Пример использования:
 * <code>
 * $WG = new mxl_widget_combobox('countries25', 'countriesWhereLess50');
 * </code>
 * 
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_widget_combobox extends mxl_widget_widget{
	/**
	 * Определяет имя файла контролера, которому будут посылаться запросы. 
	 */
	const fileNameRequestPhpFile = 'getcombobox.php';

	/**
	 * Имя комбобокса на основании, которого строится виджет. Имя служит для 
	 * получения данных от модели комбобокса.
	 * @var string
	 */
	private $comboName;
	
	/**
	 * Имя id визуального компонента в который будет сохранен результат выбора 
	 * комбобокса, если не указан, то совпадает с именем комбобокса.
	 * @var string
	 */
	private $hiddenName;
	
	/**
	 * Модель комбобокса. Определяется в конструкторе по {@link $comboName}
	 * @var mxl_dict_combobox
	 */
	private $linkCFG;
	
	/**
	 * Начальное значение ключевого поля комбобокса
	 * @var unknown_type
	 */
	private $startValue;
	
	/**
	 * Динамический фильтр. Передается через baseParams.
	 * @var string
	 */
	private $wc;
	
	/**
	 * Определяет будет ли загрузка происходить автоматически или вручную
	 * @var boolean 
	 */
	private $autoload;
	
	/**
	 * Конструктор класса виджета комбобокса.
	 * @param string $name - имя виджета
	 * @param string $comboName - имя модели комбобокса
	 * @param string $startValue - начальное значение ключевого поля
	 * @param string $hiddenName - имя id элемента в которое будет сохранено
	 * 		результирующее значение ключевого поля
	 * @param boolean $autoload - Определяет будет ли загрузка происходить
	 * 		автоматически или вручную
	 * @return mxl_widget_combobox
	 */
	function __construct($name, $comboName, $startValue = '', $params = '', 
			$hiddenName = '', $autoload = true){
		parent::__construct($name);
		$this->comboName  = $comboName;
		$this->hiddenName = $hiddenName;
		$this->autoload   = $autoload;
		
		$MDS = mxl_core_metadatas::instance();
		if ($params != '')
			$MDS->setCurrParams($params);
		else
			$MDS->setCurrParamsToEmpty();
		$this->setParams($MDS->getCurrParams());

		$this->linkCFG = $MDS->getMetaData(mxl_core_metadatas::$combobox, 
				$comboName);

		if ($startValue != '')
			$this->setStartValue($startValue);
	}
	
	/**
	 * Функция для установки начального значения комбобокса.
	 * @param any $startValue - начальное значение ключевого поля
	 */
	public function setStartValue($startValue){
		$this->startValue = $startValue;
		/*
		if (isset($startDisplay) && $startDisplay != '')
			$this->startDisplay = $startDisplay;
		else
			$this->startDisplay = $this->linkCFG->getStartDisplay($startValue);
		*/
	}

	/**
	 * Функция для получения начального значения ключевого поля
	 * @return unknown_type
	 */
	public function getStartValue(){
		return $this->startValue;
	} 
	
	/**
	 * Функция для получения начального значения отображаемого поля.
	 * @return unknown_type
	 */
	public function getStartDisplay(){
		return $this->startDisplay;
	} 
	
	/**
	 * Функция для получения имени модели комбобокса
	 * @return string
	 */
	public function getComboName(){
		return $this->comboName;
	} 
	
	/**
	 * Функция для получения имени id - элемента в которое будет сохранено
	 * результирующее значение ключевого поля
	 * @return string
	 */
	public function getHiddenName(){
		return $this->hiddenName;
	} 
	
	/**
	 * Функция для получения объекта модели комбобокса.
	 * @return mxl_dict_combobox
	 */
	public function getLinkCFG(){
		return $this->linkCFG;
	} 
	
	/**
	 * Функция для определения полного url адресса для посылки запросов
	 * от клиентсокй части к серверной. 
	 * @return string
	 */
	public function getUrlRequestPhpFile(){
		$url = mxl_cfg_url::$metaxlib.'widget'.mxl_cfg_url::$separator
				.'request'.mxl_cfg_url::$separator
				.mxl_widget_combobox::fileNameRequestPhpFile;
		return $url;
	}
	
	/**
	 * Функция для получения ajax кода инициализации и работы компонента.
	 * Генерируемый код делится на несколько частей, которые будут размещены в 
	 * определенном порядке после генерации всего ajax кода.
	 * 
	 * @param string $declarePartCommon - декларативная общая часть, размещается 1-ой.
	 * @param string $declarePartPers - декларативная часть для компонента, размещается 2-ой
	 * @param string $runPartFirst - часть которая будет выполнена 1-ой, размещается 3-ей, 
	 * 		но 1-ой в части выполнения startAutoGenScript
	 * @param string $runPartLast - часть которая будет выполнена последней, размещается 4-ей, 
	 * 		но 2-ой в части выполнения startAutoGenScript
	 * @param string $funcsAfterInit - функции которые будут выполнены поле инициализации,
	 * 		размещается 4-ой.
	 * @param boolean $append_SID - параметр (костыль) определяющий передавать ли $_SID через
	 * 		baseParam и необходимо ли определять функцию для получения _SID
	 */
	public function getJSCode(&$declarePartCommon, &$declarePartPers, 
			&$runPartFirst, &$runPartLast, &$funcsAfterInit, $append_SID = false){
		global $_SID;
		
		$MDS = mxl_core_metadatas::instance();
		
		$nameWG   = $this->getName();
		$nameCFG  = $this->getComboName();
		$CBX	  = $this->getLinkCFG();
		
		$url      = $this->getUrlRequestPhpFile();
		
		$nm1 = 'store_cbx_'.$nameWG;
		$nm2 = 'cbx_'.$nameWG;
		
		$x = ''; 
		$y = '';
		
		$bparam  = '{comboName: "'.$nameCFG.'"'; 
		if (! $MDS->getCurrParams()->isEmpty())
			$bparam .= ', params:'.$MDS->getCurrParams()->saveToJSON();
		$bparam .= ($append_SID ? ', sid:"'. $_SID.'"'     : '');
		$bparam .= ($this->wc   ? ',  wc:"'. $this->wc.'"' : ''); 
		$bparam .= '}';

		$y .= 'var '. $nm1.';'.NL;
		$x .= mxl_widget_extjs::get_combo_data_Store($nm1, $CBX, $bparam, $url,
				$this->autoload);
		
		if ($this->getStartValue()){
			$x .= mxl_widget_extjs::getFuncText_cbxOnLoad($nm1, $nm2, $this).NL;
			$x .= $nm1.'.addListener("load", strgCbxOnLoad_'.$nm1.');'.NL;
		}
		if ($this->autoload){
			if ($CBX->isInputTextMode() || ($this->getStartValue()))
				$x .= $nm1.'.load();'.NL;
		}
		
		$y .= 'var '. $nm2.';'.NL;

		$x .= mxl_widget_extjs::get_form_ComboBox($nm2, $nm1, $CBX, 
				($this->getHiddenName()?$this->getHiddenName():$nameWG),
				false);

		$declarePartPers  = $y;
		$runPartFirst = $x;
	} 
	
	/**
	 * Функция для получения ajax кода инициализации и работы компонента, в версии
	 * встраеваемой в грид.
	 * Генерируемый код делится на несколько частей, которые будут размещены в 
	 * порядке определяемом в гриде.
	 * 
	 * @param string $storage_var - часть определяющая зранилище комбобокса
	 * @param string $listeners - часть определяющая инициализацию событий
	 * @param string $listeners_desc - часть определяющая задание событий
	 * @param string $gridValueFld - имя поля грида со значением
	 * @param string $dbgrid_name - имя грида к которому привязывается комбобокс
	 * @param string $fieldname - имя поля грида к которому привязывается комбобокс
	 * @param string $edit_name - имя редактора поля
	 * @param boolean $autoload - выполнять автозагрузку значений комбобокса
	 * @param boolean $append_SID - использовать SID в baseParams
	 * @param int $listWidth - ширина списка комбобокса, если 0 то остается по умолчанию
	 */
	public function getJSCodeGridVers(&$storage_var, &$listeners, &$listeners_desc,
			$gridValueFld, $dbgrid_name, $fieldname, $edit_name, $autoload = false, 
			$append_SID = false, $listWidth = 0){
		global $_SID;
		
		$MDS = mxl_core_metadatas::instance();
		
		$nameWG	  = $fieldname;
		$nameCFG  = $this->getComboName();
		$CBX	  = $this->getLinkCFG();
		
		$url      = $this->getUrlRequestPhpFile();
		
		$nm1 = 'store_dbgcbx_'.$dbgrid_name.'_'.$nameWG;
		$nm2 = 'dbgcbx_'.$dbgrid_name.'_'.$nameWG;
		
		$x = ''; 
		$y = '';

		$bparam  = '{comboName: "'.$nameCFG.'"'; 
		if (! $MDS->getCurrParams()->isEmpty())
			$bparam .= ', params:'.$MDS->getCurrParams()->saveToJSON();
		$bparam .= ($append_SID ? ', sid:"'. $_SID.'"'     : '');
		$bparam .= ($this->wc   ? ',  wc:"'. $this->wc.'"' : ''); 
		$bparam .= '}';
		
		$x .= mxl_widget_extjs::get_combo_data_Store($nm1, $CBX, $bparam, $url, $autoload).NL;
		if ($CBX->isInputTextMode())
			$x .= $nm1.'.load();'.NL;
		$y .= mxl_widget_extjs::get_form_ComboBox($nm2, $nm1, $CBX, '', true, $listWidth);		
				
		$listeners  = $edit_name.'.addListener(\'select\', onSelect_'.$nm2.');'.NL;
		
		if ($CBX->isClearedMode())
			$listeners .= $edit_name.'.addListener(\'cleared\', onSelect_'.$nm2.');'.NL;
		 
		$listeners_desc = NL. mxl_widget_extjs::getFuncText_cbxOnSelect($nm2, 
				$dbgrid_name, $gridValueFld, $edit_name);
		
		$storage_var .= $x;
		return $y;
	}
	
	/**
	 * Метод устанавливает динамический фильтр.
	 * @param string $wc - имена полей и операторов должны быть в нижнем регистре
	 */
	public function setWC($wc){
		$this->wc = $wc;
	}
	
	/**
	 * Функция проверки типа обработчика события визуального компонента.
	 * В случае удачной проверки возвращает true;
	 * @param string $type - проверяемый тип
	 * @return boolean
	 */
	public function checkEventHandlerType($type){
		return  false;		
	}
}
?>