<?php
/**
 * Файл для описания класса визуального компонента "Дата-время", 
 * {@link mxl_widget_datetime}.
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания визуального компонента "Дата-время".
 * 
 * Пример использования:
 * <code>
 * $WG1 = new mxl_widget_datetime('start', mktime( 0, 0,0,date('m'),date('d'),date('Y')));
 * $WG2 = new mxl_widget_datetime('end');
 * </code>
 * 
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_widget_datetime extends mxl_widget_widget{
	/**
	 * Внутрення переменная для хранения текущей даты перед построением 
	 * визуальной части
	 * @var int
	 */
	private $dateTime;
	
	/**
	 * Формат отображения Даты.
	 * @var string
	 */
	private $formatDate;
	
	/**
	 * Формат отображения времени.
	 * @var string
	 */
	private $formatTime;
	
	/**
	 * Имя компонента даты-время, которым ограничено значение текущего 
	 * компонента "с начала"
	 * @var string
	 */
	private $startRangeName;
	/**
	 * Имя компонента даты-время, которым ограничено значение текущего 
	 * компонента "с конца"
	 * @var string
	 */
	private $endRangeName;
	
	/**
	 * Конструктор класса.
	 * @param string $name - имя виджета
	 * @param int $intDateTime - начальное значение даты времени
	 * @return mxl_widget_datetime
	 */
	function __construct($name, $intDateTime = 0){
		parent::__construct($name);
		$this->setDateTime($intDateTime);
		$this->formatTime = mxl_cfg_base::$format__time;
		$this->formatDate = mxl_cfg_base::$format__date;
	}
	
	/**
	 * Функция для получения ajax кода инициализации и работы компонента.
	 * Генерируемый код делится на несколько частей, которые будут размещены в 
	 * определенном порядке после генерации всего ajax кода.
	 * 
	 * @param string $declarePartCommon - декларативная общая часть, размещается 1-ой.
	 * @param string $declarePartPers - декларативная часть для компонента, размещается 2-ой
	 * @param string $runPartFirst - часть которая будет выполнена 1-ой, размещается 3-ей, 
	 * 		но 1-ой в части выполнения startAutoGenScript
	 * @param string $runPartLast - часть которая будет выполнена последней, размещается 4-ей, 
	 * 		но 2-ой в части выполнения startAutoGenScript
	 * @param string $funcsAfterInit - функции которые будут выполнены поле инициализации,
	 * 		размещается 4-ой.
	 * @param boolean $append_SID - параметр (костыль) определяющий передавать ли $_SID через
	 * 		baseParam и необходимо ли определять функцию для получения _SID
	 */
	public function getJSCode(&$declarePartCommon, &$declarePartPers, 
			&$runPartFirst, &$runPartLast, &$funcsAfterInit, $append_SID = false){
		
		$find_wgDateTime_Validate = false;

		$x  = '';
		$y  = '';
		$range = '';
		
		$nameWG = $this->getName();
		$date   = $this->getDateJS();
		$time   = $this->getTimeJS();
		
		$nm_d = 'date_field_'.$nameWG;
		$nm_t = 'time_field_'.$nameWG;
		
		if ($this->getStartRangeName() && $this->getStartRangeName() != ''){
			$range = 'startDateField: "ext_date_field_'
					.$this->getStartRangeName().'",'.NL;
			$find_wgDateTime_Validate = true;
		}

		if ($this->getEndRangeName() && $this->getEndRangeName() != ''){
			$range = 'endDateField: "ext_date_field_'.$this->getEndRangeName()
					.'",'.NL;
			$find_wgDateTime_Validate = true;
		}

		$y .= 'var '. $nm_d.';'.NL;
		$x .= $nm_d.' = new Ext.form.DateField({'.NL
				.'  renderTo: "'.$nm_d.'",'.NL
				.'  id: "ext_'.$nm_d.'",'.NL
                .'  width: 95,'.NL
				//.'  autoWidth: true,'.NL
				.($range ? '  vtype: "daterange",'.NL : '')
				.($range ? '  '.$range	: '')
				.'  format: "'.$this->formatDate.'",'.NL
				.'  allowBlank: false,'.NL
				.'  anchor : "32%"'.NL
				.'  });'.NL;
		  
		$strtime = '';
		if (isset($time) && $time != '')
			$strtime = 'value: "'.$time.'",'.NL;
				
		$y	.= 'var '. $nm_t.';'.NL;
		$x	.= $nm_t.' = new Ext.form.TimeField({'.NL
				.'  renderTo: "'.$nm_t.'",'.NL
				.'  id: "ext_'.$nm_t.'",'.NL
				.'  width: 60,'.NL
				.($strtime ? '  '.$strtime  : '')
				.'  format: "'.$this->formatTime.'",'.NL
				.'  allowBlank: false,'.NL
				.'  anchor : "32%"'.NL
				.'  });'.NL;
	
		if (($find_wgDateTime_Validate) && 
				(strpos($declarePartCommon, 'Ext.apply(Ext.form.VTypes, {'.NL
						.'	   daterange : function(val, field)') === false)){
			$declarePartCommon .= ''.NL
					.'  Ext.form.Field.prototype.msgTarget = \'side\';'.NL.NL
					.'  Ext.apply(Ext.form.VTypes, {'.NL
					.'	   daterange : function(val, field) {'.NL
					.'	     var date = field.parseDate(val);'.NL.NL
					.'	     if(!date){'.NL
					.'	     return;'.NL
					.'	     }'.NL
					.'	     if ( field.startDateField && '.NL 
					.'           (!this.dateRangeMax || '.NL 
					.'            (date.getTime() != this.dateRangeMax.getTime()))){'.NL
					.'	       var start = Ext.getCmp(field.startDateField);'.NL
					.'	       start.setMaxValue(date);'.NL
					.'        start.validate();'.NL
					.'        this.dateRangeMax = date;'.NL
					.'      } '.NL
					.'      else if (field.endDateField '.NL
					.'            && (!this.dateRangeMin ||'.NL
					.'                (date.getTime() != this.dateRangeMin.getTime()))){'.NL
					.'        var end = Ext.getCmp(field.endDateField);'.NL
					.'	       end.setMinValue(date);'.NL//!!!
					.'        end.validate();'.NL		 //!!!
					.'        this.dateRangeMin = date;'.NL
					.'      }'.NL
					.'      /*'.NL
					.'       * Always return true since we\'re only using this vtype to set the'.NL
					.'       * min/max allowed values (these are tested for after the vtype test)'.NL
					.'       */'.NL
					.'      return true;'.NL
					.'    }'.NL
					.'  });';
		}
		if ($date != '')
			$x1 = '  '.$nm_d.'.setValue("'.$date.'");'.NL;
		
		$declarePartPers  = $y; 
		$runPartFirst     = $x;
		$runPartLast      = $x1;
	}
	
	/**
	 * Функция получения имени компонента, который будет ограничивать 
	 * значение текущего компонента "с начала"
	 * @return string
	 */
	public function getStartRangeName(){
		return $this->startRangeName;
	}
	
	/**
	 * Функция для установки имени компонента, который будет ограничивать 
	 * значение текущего компонента "с начала"
	 * @param string $startRangeName
	 */
	public function setStartRangeName($startRangeName){
		$this->startRangeName = $startRangeName;
	}
	
	/**
	 * Функция получения имени компонента, который будет ограничивать 
	 * значение текущего компонента "с конца"
	 * @return string
	 */
	public function getEndRangeName(){
		return $this->endRangeName;
	}
	
	/**
	 * Функция для установки имени компонента, который будет ограничивать 
	 * значение текущего компонента "с конца"
	 * @param string $startRangeName
	 */
	public function setEndRangeName($endRangeName){
		$this->endRangeName = $endRangeName;
	}
	
	/**
	 * Метод для получения текущего значения даты
	 * @return string
	 */
	public function getDate(){
		$res = $this->dateTime;
		return strftime('%m/%d/%Y', $res);
	}
	
	/**
	 * Метод для получения текущего значения даты для формирования JS кода 
	 * с учетом смещения по времени
	 * @return string
	 */
	private function getDateJS(){
		$res = $this->dateTime;
		if ($this->getTimeShift())
			$res = mxl_utils_edatetime::applyTimeShift($res, $this->getTimeShift());
		return strftime('%m/%d/%Y', $res);
	}
	
	/**
	 * Метод для получения текущего значения времени
	 * @return int
	 */
	public function getTime(){
		$res = $this->dateTime;
		return strftime('%H:%M', $res);
	}
	
	/**
	 * Метод для получения текущего значения времени для формирования JS кода 
	 * с учетом смещения по времени
	 * @return string
	 */
	public function getTimeJS(){
		$res = $this->dateTime;
		 if ($this->getTimeShift())
			$res = mxl_utils_edatetime::applyTimeShift($res, $this->getTimeShift());
		return strftime('%H:%M', $res);
	}
	
	/**
	 * Метод для установки текущего значения даты и времени
	 * @param int $intDateTime
	 */
	public function setDateTime($intDateTime){
		$this->dateTime = $intDateTime;
	}
	
	/**
	 * Метод установки формата отображения Даты.
	 * @param string $formatDate
	 */
	public function setFormatDate($formatDate){
		$this->formatDate = $formatDate;
	}
	
	/**
	 * Метод установки формата отображения Времени.
	 * @param string $formatTime
	 */
	public function setFormatTime($formatTime){
		$this->formatTime = $formatTime;
	}

	/**
	 * Функция проверки типа обработчика события визуального компонента.
	 * В случае удачной проверки возвращает true;
	 * @param string $type - проверяемый тип
	 * @return boolean
	 */
	public function checkEventHandlerType($type){
		return  false;		
	}
}
?>