<?php
/**
 * Файл для описания класса визуального компонента грид, 
 * {@link mxl_widget_dbgrid}.
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания виждета грид.
 * 
 * Пример использования:
 * <code>
 * $WG1 = new mxl_widget_dbgrid(  'gbg27',       'loc1');
 * $WG2 = new mxl_widget_dbgrid(  'gbg28',       'fees1');
 * </code>
 * 
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_widget_dbgrid extends mxl_widget_widget{
	/**
	 * Определяет имя файла контролера, которому будут посылаться запросы. 
	 */
	const fileNameRequestPhpFile = 'getdbgrid.php';
	
	/**
	 * Тип обработчика события - нажатие кнопки обновить (изменение данных) 
	 */
	public static $ehtRefreshData = 'RefreshData';
	
	/**
	 * Тип обработчика события - нажатие кнопки добавить (добавление новой строки) 
	 */
	public static $ehtAppendRow = 'AppendRow';
	
	/**
	 * Тип обработчика события - нажатие кнопки Сохранить (сохранить изменение данных) 
	 */
	public static $ehtSaveData = 'SaveData';
		
	/**
	 * Тип обработчика события - нажатие кнопки удалить (удалить выделенную строку) 
	 */
	public static $ehtDeleteRow = 'DeleteRow';
	
	/**
	 * Имя грида на основании, которого строится виджет. Имя служит для 
	 * получения данных от модели грида.
	 * @var string
	 */
	private $dbgridName;
	
	/**
	 * Модель грида. Определяется в конструкторе по {@link $dbgridName}
	 * @var mxl_dict_dbgrid
	 */
	private $linkCFG;
	
	/**
	 * Динамический фильтр. Передается через baseParams.
	 * @var string
	 */
	private $wc;

	/**
	 * Суффикс, который будет добавлен к имени таблицы. Передается через 
	 * baseParams.
	 * @var string
	 */
	private $tableNameSuf;
	
	/**
	 * Определяет будет ли загрузка происходить автоматически или вручную
	 * @var boolean 
	 */
	private $autoload;
	
	/**
	 * Конструктор класса
	 * @param string $name - имя виджета
	 * @param string $dbgridName - имя модели грида
	 * @param mxl_dict_dbgrids $DBGS - хранилище гридов, если не определно
	 * 		то в конструкторе создается временное для загрузки модели грида.
	 * @param boolean $autoload - Определяет будет ли загрузка происходить
	 * 		автоматически или вручную
	 * @return mxl_widget_dbgrid
	 */
	function __construct($name, $dbgridName, $params = '', $autoload = true){
		parent::__construct($name);
		
		$this->dbgridName = $dbgridName;
		$this->autoload   = $autoload;

		$MDS = mxl_core_metadatas::instance();
		if ($params != '')
			$MDS->setCurrParams($params);
		else
			$MDS->setCurrParamsToEmpty();
		$this->setParams($MDS->getCurrParams());
					
		$this->linkCFG = $MDS->getMetaData(mxl_core_metadatas::$dbgrid, 
				$dbgridName);
	}
	
	/**
	 * Метод для получения имени модели грида
	 * @return string
	 */
	public function getDBGridName(){
		return $this->dbgridName;
	} 
	
	/**
	 * Метод для получения модели грида
	 * @return mxl_dict_dbgrid
	 */
	public function getLinkCFG(){
		return $this->linkCFG;
	} 
	
	/**
	 * Функция для определения полного url адресса для посылки запросов
	 * от клиентсокй части к серверной. 
	 * @return string
	 */
	public function getUrlRequestPhpFile(){
		$url = mxl_cfg_url::$metaxlib.'widget'.mxl_cfg_url::$separator.'request'
				.mxl_cfg_url::$separator.mxl_widget_dbgrid::fileNameRequestPhpFile;
		return $url;
	}
	
	/**
	 * Функция для получения ajax кода инициализации и работы компонента.
	 * Генерируемый код делится на несколько частей, которые будут размещены в 
	 * определенном порядке после генерации всего ajax кода.
	 * 
	 * @param string $declarePartCommon - декларативная общая часть, размещается 1-ой.
	 * @param string $declarePartPers - декларативная часть для компонента, размещается 2-ой
	 * @param string $runPartFirst - часть которая будет выполнена 1-ой, размещается 3-ей, 
	 * 		но 1-ой в части выполнения startAutoGenScript
	 * @param string $runPartLast - часть которая будет выполнена последней, размещается 4-ей, 
	 * 		но 2-ой в части выполнения startAutoGenScript
	 * @param string $funcsAfterInit - функции которые будут выполнены поле инициализации,
	 * 		размещается 4-ой.
	 * @param boolean $append_SID - параметр (костыль) определяющий передавать ли $_SID через
	 * 		baseParam и необходимо ли определять функцию для получения _SID
	 */
	public function getJSCode(&$declarePartCommon, &$declarePartPers, 
			&$runPartFirst, &$runPartLast, &$funcsAfterInit, $append_SID = false){
		global $_SID;
		
		$MDS = mxl_core_metadatas::instance();
		
		$nameWG   = $this->getName();
		$nameCFG  = $this->getDBGridName();
		$DBG	  = $this->getLinkCFG();
		
		/*
		 * Пока необходимо только для итогов по гриду, при первоначальной 
		 * инициализации
		 */
		$DBG->setWC($this->wc);
		
		$url      = $this->getUrlRequestPhpFile();
		
		$dbgName  = $DBG->getName();
	
		$befX = '';
		$x    = '';
		$y    = '';
		$record_vars = '';
		$edir_vars	 = '';
		
		
		$bparam  = '{gridName: "'. $nameCFG .'", pkey: "'. $DBG->getPrimKey().'"';
		if (! $MDS->getCurrParams()->isEmpty())
			$bparam .= ', params:'.$MDS->getCurrParams()->saveToJSON();
		$bparam .= ($append_SID ? ', sid:"'. $_SID.'"'     : '');
		$bparam .= ($this->wc   ? ',  wc:"'. $this->wc.'"' : ''); 
		$bparam .= ($this->getTimeShift() ? ',  timeShift:"'. $this->getTimeShift().'"' : ''); 
		$bparam .= ($this->tableNameSuf ?', table_name_suf:"'. $this->tableNameSuf.'"':'');		
		$bparam .= '}';
		
		$nm1 = 'store_dbg_'.$nameWG;
		$nm2 = 'dbg_'.$nameWG;
		
		$y .= 'var '. $nm1.';'.NL;
		$x .= mxl_widget_extjs::get_dbg_data_Store($nm1, $DBG, $bparam, $url,
				$this->getAutoload()).NL;
		
		if ($DBG->isTotalsFooterMode())
			$x .= 'var '.$nm2.'_summary = new Ext.ux.grid.GridSummary();'.NL;
		
		$y .= 'var '. $nm2.';'.NL;
		$x .= mxl_widget_extjs::get_dbg_grid_GridPane($this, $nm2, $nm1, $DBG, $url).NL;
		
		
		$x .= mxl_widget_extjs::getFuncText_dbgUpdateGridInformation($nm2, $DBG).NL;
		
		if ($DBG->isTotalsGridMode())
			$x .= mxl_widget_extjs::get_dbg_grid_initTotalStr($nm2, $DBG).NL;
			
		if ($DBG->isTotalsFooterMode())
			$befX .= mxl_widget_extjs::get_dbg_gridFooterTotals_sumRenderers($nm2,$DBG);

		$declPP = $runP = '';
		if (mxl_widget_extjs::getJS_dbg_Editors($declarePartCommon, $declPP, $runP,
				$nameWG, $DBG, $this->getParams(), $url, $this->getAutoload())){
			$y .= $declPP.NL;
			$x .= $runP;
		}

		if (!$DBG->isReadOnly()) {
			$sss = '';
			$y .= mxl_widget_extjs::getJS_dbg_EditRow($nameWG, $DBG,
                    $this->getParams(), $url);
			
			$edtPrmExt = '';
        	if ($append_SID)
        		$edtPrmExt .= ',sid:"'. $_SID.'"';
        	$edtPrmExt .= ($this->getTimeShift() ? 
        			(($edtPrmExt!=''?', ':'').'timeShift:"'. $this->getTimeShift().'"') : 
        			'');
        	$edtFncScsExt = '';
        	if ($DBG->isTotalsGridMode())
        		$edtFncScsExt = mxl_widget_extjs::get_dbg_grid_updateTotalStr1($nm2,
        			'            ').NL;
        	
			if (! $this->hasEventHandler(mxl_widget_dbgrid::$ehtAppendRow)){
        		$DBG->preprareDefaults();
				$sss .= mxl_widget_extjs::getFuncText_dbgAppendRow($nameWG, 
						'record_'.$nameWG, $DBG->getDefaults()).NL; 
			}
			$sss .= mxl_widget_extjs::getFuncText_grdOnAfterEdit($nm2, $DBG, $url);
            $sss .= mxl_widget_extjs::getFuncText_dbgOnUpdate($nm2);
        		  
			if (! $this->hasEventHandler(mxl_widget_dbgrid::$ehtDeleteRow))
				$sss .= mxl_widget_extjs::getFuncText_dbgDeleteRow('_'.$nameWG, 
        				$url, $edtPrmExt, $nm2, $edtFncScsExt).NL;
			if (! $this->hasEventHandler(mxl_widget_dbgrid::$ehtRefreshData))
        		$sss .= mxl_widget_extjs::getFuncText_dbgRefreshData('_'.$nameWG, $nm1).NL;
        	
       		if (! $this->hasEventHandler(mxl_widget_dbgrid::$ehtSaveData)){
        		if( $DBG->isManyRowsSaveMode())
        			$sss .= mxl_widget_extjs::getFuncText_dbgSaveRow_ManyRows_array(
        					'_'.$nameWG, $url, $edtPrmExt, $edtFncScsExt).NL;
        		else
        			$sss .= mxl_widget_extjs::getFuncText_dbgSaveRow('_'.$nameWG, $url, 
        					$edtPrmExt, $edtFncScsExt).NL;
       		}
        	$x .= $sss;
        }
        
		if (strpos($funcsAfterInit, "Ext.util.Format.comboRenderer = function(combo)") === false)
			$funcsAfterInit .= NL.NL.
				'Ext.util.Format.comboRenderer = function(combo) {'.NL.
				'	return function(value) {'.NL.
				'		var record = combo.findRecord(combo.valueField, value);'.NL.
				'		return record ? record.get(combo.displayField) : combo.valueNotFoundText;'.NL.
				'	}'.NL.
				'}'.NL.NL;
		
		if( (!$DBG->isManyRowsSaveMode()) &&
				(strpos($declarePartCommon, "function dbgBeforeRowSelect(sm, rowIndex, '
						.'keepExisting, record)") === false))
		    $declarePartCommon .= NL.mxl_widget_extjs::getFuncText_dbgBeforeRowSelect('').NL;    

        $declarePartPers = $y;
        $runPartFirst    = $befX . $x;
	}
	
	/**
	 * Метод устанавливает динамический фильтр.
	 * @param $wc - sql string. имена полей и операторов должны быть в нижнем регистре
	 */
	public function setWC($wc){
		$this->wc = $wc;
	}
	
	/**
	 * Метод устанавливающий имя суффикса таблицы.
	 * @param $tableNameSuf - имя суффикса таблицы
	 */
	public function setTableNameSuf($tableNameSuf){
		$this->tableNameSuf = $tableNameSuf;
	}
	
	/**
	 * Функция проверки типа обработчика события визуального компонента.
	 * В случае удачной проверки возвращает true;
	 * @param string $type - проверяемый тип
	 * @return boolean
	 */
	public function checkEventHandlerType($type){
		return  ($type == mxl_widget_dbgrid::$ehtAppendRow) || 
				($type == mxl_widget_dbgrid::$ehtDeleteRow) || 
				($type == mxl_widget_dbgrid::$ehtRefreshData) || 
				($type == mxl_widget_dbgrid::$ehtSaveData);
		
	}
	
	/**
	 * Определяет будет ли загрузка происходить автоматически или вручную
	 * Устанавливается в конструкторе грида
	 * @return boolean
	 */
	public function getAutoload(){
		return $this->autoload;
	}
}
?>